'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var React = require('react');
var domAlign = require('dom-align');
var RcUtil = require('rc-util');
var KeyCode = RcUtil.KeyCode;
var Dom = RcUtil.Dom;
var assign = require('object-assign');
var _anim = require('css-animation');

function prefixClsFn(prefixCls) {
  var args = Array.prototype.slice.call(arguments, 1);
  return args.map(function (s) {
    if (!s) {
      return prefixCls;
    }
    return prefixCls + '-' + s;
  }).join(' ');
}

function buffer(fn, ms) {
  var timer;
  return function () {
    if (timer) {
      clearTimeout(timer);
    }
    timer = setTimeout(fn, ms);
  };
}

var Dialog = React.createClass({
  displayName: 'Dialog',

  align: function align() {
    var align = this.props.align;
    domAlign(React.findDOMNode(this.refs.dialog), align.node || window, align);
  },

  monitorWindowResize: function monitorWindowResize() {
    if (!this.resizeHandler) {
      this.resizeHandler = Dom.addEventListener(window, 'resize', buffer(this.align, 80));
    }
  },

  anim: function anim(el, transitionName, animation, enter, fn) {
    var props = this.props;
    if (!transitionName && animation) {
      transitionName = props.prefixCls + '-' + animation;
    }
    if (transitionName) {
      _anim(el, transitionName + (enter ? '-enter' : '-leave'), fn);
    } else if (fn) {
      fn();
    }
  },

  unMonitorWindowResize: function unMonitorWindowResize() {
    if (this.resizeHandler) {
      this.resizeHandler.remove();
      this.resizeHandler = null;
    }
  },

  componentDidMount: function componentDidMount() {
    this.componentDidUpdate();
  },

  componentDidUpdate: function componentDidUpdate(prevProps) {
    var _this = this;

    var props = this.props;
    var dialogDomNode = React.findDOMNode(this.refs.dialog);
    var maskNode = React.findDOMNode(this.refs.mask);
    prevProps = prevProps || {};
    if (props.visible) {
      this.monitorWindowResize();
      // first show
      if (!prevProps.visible) {
        this.align();
        this.anim(maskNode, props.maskTransitionName, props.maskAnimation, true);
        this.anim(dialogDomNode, props.transitionName, props.animation, true, function () {
          props.onShow();
        });
        this.lastOutSideFocusNode = document.activeElement;
        dialogDomNode.focus();
      } else if (props.align !== prevProps.align) {
        this.align();
      }
    } else {
      if (prevProps.visible) {
        this.anim(maskNode, props.maskTransitionName, props.maskAnimation);
        this.anim(dialogDomNode, props.transitionName, props.animation, false, function () {
          props.onClose();
          if (props.mask && _this.lastOutSideFocusNode) {
            try {
              _this.lastOutSideFocusNode.focus();
            } catch (e) {}
            _this.lastOutSideFocusNode = null;
          }
        });
      }
      this.unMonitorWindowResize();
    }
  },

  componentWillUnmount: function componentWillUnmount() {
    this.unMonitorWindowResize();
  },

  handleKeyDown: function handleKeyDown(e) {
    var props = this.props;
    if (props.closable) {
      if (e.keyCode === KeyCode.ESC) {
        this.props.onRequestClose();
      }
    }
    // keep focus inside dialog
    if (props.visible) {
      if (e.keyCode === KeyCode.TAB) {
        var activeElement = document.activeElement;
        var dialogRoot = React.findDOMNode(this.refs.dialog);
        var sentinel = React.findDOMNode(this.refs.sentinel);
        if (e.shiftKey) {
          if (activeElement === dialogRoot) {
            sentinel.focus();
          }
        } else if (activeElement === React.findDOMNode(this.refs.sentinel)) {
          dialogRoot.focus();
        }
      }
    }
  },

  handleMaskClick: function handleMaskClick() {
    if (this.props.closable) {
      this.props.onRequestClose();
    }
    React.findDOMNode(this.refs.dialog).focus();
  },

  render: function render() {
    var props = this.props;
    var visible = props.visible;
    var prefixCls = props.prefixCls;
    var className = [prefixClsFn(prefixCls, 'wrap')];
    var closable = props.closable;
    if (!visible) {
      className.push(prefixClsFn(prefixCls, 'wrap-hidden'));
    }
    var dest = {};
    if (props.width !== undefined) {
      dest.width = props.width;
    }
    if (props.height !== undefined) {
      dest.height = props.height;
    }
    if (props.zIndex !== undefined) {
      dest.zIndex = props.zIndex;
    }

    var style = assign({}, props.style, dest);

    var maskProps = {
      onClick: this.handleMaskClick
    };
    var dialogProps = {
      className: [prefixCls, props.className].join(' '),
      tabIndex: '0',
      role: 'dialog',
      ref: 'dialog',
      style: style,
      onKeyDown: this.handleKeyDown
    };

    if (style.zIndex) {
      maskProps.style = { zIndex: style.zIndex };
    }
    var footer;
    if (props.footer) {
      footer = React.createElement(
        'div',
        { className: prefixClsFn(prefixCls, 'footer') },
        props.footer
      );
    }
    var header;
    if (props.title || closable) {
      header = React.createElement(
        'div',
        { className: prefixClsFn(prefixCls, 'header') },
        closable ? React.createElement(
          'a',
          { tabIndex: '0', onClick: props.onRequestClose, className: [prefixClsFn(prefixCls, 'close')].join('') },
          React.createElement('span', { className: prefixClsFn(prefixCls, 'close-x') })
        ) : null,
        React.createElement(
          'div',
          { className: prefixClsFn(prefixCls, 'title') },
          props.title
        )
      );
    }
    return React.createElement(
      'div',
      { className: className.join(' ') },
      props.mask ? React.createElement('div', _extends({}, maskProps, { className: prefixClsFn(prefixCls, 'mask'), ref: 'mask' })) : null,
      React.createElement(
        'div',
        dialogProps,
        React.createElement(
          'div',
          { className: prefixClsFn(prefixCls, 'content') },
          header,
          React.createElement(
            'div',
            { className: prefixClsFn(prefixCls, 'body') },
            props.children
          ),
          footer
        ),
        React.createElement(
          'div',
          { tabIndex: '0', ref: 'sentinel', style: { width: 0, height: 0, overflow: 'hidden' } },
          'sentinel'
        )
      )
    );
  }
});

module.exports = Dialog;

// empty